@@ -18,30 +18,149 @@
 
 #include "config.h"
 
+#include <gconf/gconf-client.h>
 #include <ctype.h>
 #include <gnome.h>
 
 #include "gnome-mud.h"
 
 static char const rcsid[] =
-    "$Id: profiles.c,v 1.20 2002/02/28 13:40:17 lobbin Exp $";
+    "$Id: profiles.c,v 1.32 2003/06/24 14:55:47 jordim Exp $";
 
 WIZARD_DATA2 *connections_find_by_profilename(gchar *name);
 gint 		  get_size(GtkCList *clist);
-gint		  g_list_compare_strings(gchar *a, gchar *b);
 
-GList	  *ProfilesList;
-GList	  *ProfilesData;
-GList	  *Profiles;
+GList     *ProfilesData;
+GList     *Profiles;
 gint	   selected, pselected;
 gint	   connection_selected;
 
 GtkWidget *main_clist;
 
+extern GConfClient	*gconf_client;
 
-gint g_list_compare_strings(gchar *a, gchar *b)
+static void profile_gconf_sync_list()
 {
-	return g_strcasecmp((gchar *) a, (gchar *) b);
+	GList *entry;
+	GSList *list = NULL;
+
+	for (entry = g_list_first(ProfilesData); entry != NULL; entry = g_list_next(entry))
+	{
+		list = g_slist_append(list, ((PROFILE_DATA *) entry->data)->name);
+	}
+
+	gconf_client_set_list(gconf_client, "/apps/gnome-mud/profiles/list", GCONF_VALUE_STRING, list, NULL);
+}
+
+static void profile_gconf_sync_connection_list()
+{
+	GList *entry;
+	GSList *list = NULL;
+
+	for (entry = g_list_first(Profiles); entry != NULL; entry = g_list_next(entry))
+	{
+		WIZARD_DATA2 *wd = (WIZARD_DATA2 *) entry->data;
+
+		if (strlen(wd->playername) > 0)
+		{
+			gchar buf[1024];
+			
+			g_snprintf(buf, 1024, "%s-at-%s", wd->playername, wd->name);
+			list = g_slist_append(list, buf);
+		}
+		else
+		{
+			list = g_slist_append(list, wd->name);
+		}
+	}
+
+	gconf_client_set_list(gconf_client, "/apps/gnome-mud/connections/list", GCONF_VALUE_STRING, list, NULL);
+}
+
+static void profile_gconf_sync_wizard(WIZARD_DATA2 *wd)
+{
+	gchar buf[1024] = "";
+	gchar *bufn;
+
+	if (strlen(wd->playername) > 0)
+	{
+		bufn = g_strdup_printf("%s-at-%s", wd->playername, wd->name);
+	}
+	else
+	{
+		bufn = g_strdup(wd->name);
+	}
+
+#define SYNC(Name, Value) \
+	g_snprintf(buf, 1024, "/apps/gnome-mud/connections/%s/%s", bufn, Name); \
+	gconf_client_set_string(gconf_client, buf, Value, NULL);
+
+	SYNC("name", wd->name);
+	SYNC("hostname", wd->hostname);
+	SYNC("port", wd->port);
+	SYNC("playername", wd->playername);
+	SYNC("password", wd->password);
+	SYNC("profile", wd->profile);
+#undef SYNC
+
+	g_free(bufn);
+}
+
+static KEYBIND_DATA *profile_load_keybinds(gchar *profile_name)
+{
+	KEYBIND_DATA *k = NULL;
+	GSList *slist, *lentry;
+	gchar pname[255];
+
+	snprintf(pname, 255, "/apps/gnome-mud/profiles/%s/keybinds", profile_name);
+	slist = gconf_client_get_list(gconf_client, pname, GCONF_VALUE_STRING, NULL);
+	for (lentry = slist; lentry != NULL; lentry = g_slist_next(lentry))
+	{
+		KEYBIND_DATA *kd = (KEYBIND_DATA *) g_malloc0(sizeof(KEYBIND_DATA));
+		int KB_state = 0;
+		gchar **a, *keyv_begin = NULL;
+			
+		a = g_strsplit((gchar *) lentry->data, "=", 2);
+
+		if (strstr(a[0], "Control"))	KB_state |= 4;
+		if (strstr(a[0], "Alt"))		KB_state |= 8;
+
+		keyv_begin = a[0] + strlen(a[0]) - 2;
+
+		while(!(keyv_begin == (a[0]-1) || keyv_begin[0] == '+')) keyv_begin--;
+		keyv_begin++;
+
+		kd->state	= KB_state;
+		kd->keyv	= gdk_keyval_from_name(keyv_begin);
+		kd->data	= g_strdup(a[1]);
+		kd->next	= k;
+		k 			= kd;
+
+		g_strfreev(a);
+	}
+
+	return k;
+}
+
+static GList *profile_load_list(gchar *pn, gchar *pt)
+{
+	GList *list = NULL;
+	GSList *slist;
+	GSList *lentry;
+	gchar pname[255];
+
+	snprintf(pname, 255, "/apps/gnome-mud/profiles/%s/%s", pn, pt);
+	slist = gconf_client_get_list(gconf_client, pname, GCONF_VALUE_STRING, NULL);
+	for (lentry = slist; lentry != NULL; lentry = g_slist_next(lentry))
+	{
+		gchar **a;
+
+		a = g_strsplit((gchar *) lentry->data, "=", 2);
+
+		list = g_list_append(list, (gpointer) a);
+	}
+
+	return list;
 }
 
 void load_profiles()
@@ -49,238 +168,50 @@ void load_profiles()
 	/*
 	 * Profiles
 	 */
-	{
-		gint nr, i;
-		gchar **profiles;
-		gnome_config_get_vector("/gnome-mud/Data/Profiles", &nr, &profiles);
-
-		if (nr == 0)
-		{
-			PROFILE_DATA *pd = g_malloc0(sizeof(PROFILE_DATA));
-			
-			pd->name		= "Default";
-			pd->alias		= (GtkCList *) gtk_clist_new(2);
-			pd->variables	= (GtkCList *) gtk_clist_new(2);
-			pd->triggers	= (GtkCList *) gtk_clist_new(2);
-			
-			ProfilesList = g_list_append(ProfilesList, "Default");
-			ProfilesData = g_list_append(ProfilesData, (gpointer) pd);
-		}
-
-		for (i = 0; i < nr; i++)
-		{
-			PROFILE_DATA *pd = g_malloc0(sizeof(PROFILE_DATA));
-			gchar name[50];
-			gint  new_nr, k;
-			gchar **vector, **vector2;
+	GSList *profiles, *connections;
+	GSList *entry;
 		
-			pd->name  	  = profiles[i];
-			pd->alias 	  = (GtkCList *) gtk_clist_new(2);
-			pd->variables = (GtkCList *) gtk_clist_new(2);
-			pd->triggers  = (GtkCList *) gtk_clist_new(2);
-			pd->kd		  = NULL;
+	profiles = gconf_client_get_list(gconf_client, "/apps/gnome-mud/profiles/list", GCONF_VALUE_STRING, NULL);
 
-			/*
-			 * Aliases
-			 */
-			g_snprintf(name, 50, "/gnome-mud/Profile: %s/Alias", profiles[i]);
-			gnome_config_get_vector(name, &new_nr, &vector);
-			g_snprintf(name, 50, "/gnome-mud/Profile: %s/AliasValue", profiles[i]);
-			gnome_config_get_vector(name, &new_nr, &vector2);
-			for (k = 0; k < new_nr; k++)
-			{
-				gchar *t[2] = { vector[k], vector2[k] };
+	for (entry = profiles; entry != NULL; entry = g_slist_next(entry))
+	{
+		PROFILE_DATA *pd = g_malloc0(sizeof(PROFILE_DATA));
 
-				if (strlen(vector[k]) < 1)
-				{
-					continue;
-				}
-				
-				gtk_clist_append(pd->alias, t);
-			}
+		pd->name = g_strdup((gchar *) entry->data);
+		pd->alias = profile_load_list(pd->name, "aliases");
+		pd->variables = profile_load_list(pd->name, "variables");
+		pd->triggers = profile_load_list(pd->name, "triggers" );
 
-			/*
-			 * Variables
-			 */
-			g_snprintf(name, 50, "/gnome-mud/Profile: %s/Variables", profiles[i]);
-			gnome_config_get_vector(name, &new_nr, &vector);
-			g_snprintf(name, 50, "/gnome-mud/Profile: %s/VariablesValue", profiles[i]);
-			gnome_config_get_vector(name, &new_nr, &vector2);
-			for (k = 0; k < new_nr; k++)
-			{
-				gchar *t[2] = { vector[k], vector2[k] };
-
-				if (strlen(vector[k]) < 1)
-				{
-					continue;
-				}
-				
-				gtk_clist_append(pd->variables, t);
-			}
-
-			/*
-			 * Triggers
-			 */
-			g_snprintf(name, 50, "/gnome-mud/Profile: %s/Triggers", profiles[i]);
-			gnome_config_get_vector(name, &new_nr, &vector);
-			g_snprintf(name, 50, "/gnome-mud/Profile: %s/TriggersValue", profiles[i]);
-			gnome_config_get_vector(name, &new_nr, &vector2);
-			for (k = 0; k < new_nr; k++)
-			{
-				gchar *t[2] = { vector[k], vector2[k] };
-
-				if (strlen(vector[k]) < 1)
-				{
-					continue;
-				}
-				
-				gtk_clist_append(pd->triggers, t);
-			}
-
-			/*
-			 * Keybinds
-			 */
-			g_snprintf(name, 50, "/gnome-mud/Profile: %s/Keybinds", profiles[i]);
-			gnome_config_get_vector(name, &new_nr, &vector);
-			g_snprintf(name, 50, "/gnome-mud/Profile: %s/KeybindValues", profiles[i]);
-			gnome_config_get_vector(name, &new_nr, &vector2);
-			for (k = 0; k < new_nr; k++)
-			{
-				KEYBIND_DATA *kd = (KEYBIND_DATA *) g_malloc0(sizeof(KEYBIND_DATA));
-				int KB_state = 0;
-				gchar *keyv_begin;
-
-				if (strlen(vector[k]) < 1)
-				{
-					continue;
-				}
-
-				if (strstr(vector[k], "Control"))	KB_state |= 4;
-				if (strstr(vector[k], "Alt"))		KB_state |= 8;
-				
-				keyv_begin = vector[k] + strlen(vector[k]) - 2;
-			
-				while (!(keyv_begin == (vector[k]-1) || keyv_begin[0] == '+')) keyv_begin--;
-				keyv_begin++;
-
-				kd->state = KB_state;
-				kd->keyv  = gdk_keyval_from_name(keyv_begin);
-				kd->data  = g_strdup(vector2[k]);
-				kd->next  = pd->kd;
-				pd->kd    = kd;
-			}
-
-			ProfilesList = g_list_append(ProfilesList, (gpointer) profiles[i]);
-			ProfilesData = g_list_append(ProfilesData, (gpointer) pd);
-		}
+		pd->kd = profile_load_keybinds(pd->name);
+	
+		ProfilesData = g_list_append(ProfilesData, (gpointer) pd);
 	}
 
 	/*
 	 * Wizard
 	 */
+	connections = gconf_client_get_list(gconf_client, "/apps/gnome-mud/connections/list", GCONF_VALUE_STRING, NULL);
+
+#define LOAD(Name, Value) \
+		g_snprintf(buf, 1024, "/apps/gnome-mud/connections/%s/%s", (gchar *) entry->data, Name); \
+		Value = g_strdup(gconf_client_get_string(gconf_client, buf, NULL));
+
+	for (entry = connections; entry != NULL; entry = g_slist_next(entry))
 	{
-		WIZARD_DATA2 *wd;
-		
-		gint i = 0;
-		gchar name[50];
-		gchar *value;
-
-		while(1)
-		{
-			g_snprintf(name, 50, "/gnome-mud/Connection%d/Name=", i);
-			value = gnome_config_get_string(name);
-
-			if (strlen(value) < 1)
-			{
-				break;
-			}
-			
-			wd = g_malloc0(sizeof(WIZARD_DATA2));
-			
-			wd->name = value;
-			
-			g_snprintf(name, 50, "/gnome-mud/Connection%d/Host=", i);
-			wd->hostname = gnome_config_get_string(name);
-			
-			g_snprintf(name, 50, "/gnome-mud/Connection%d/Port=", i);
-			wd->port = gnome_config_get_string(name);
-			
-			g_snprintf(name, 50, "/gnome-mud/Connection%d/Char=", i);
-			wd->playername = gnome_config_get_string(name);
-			
-			g_snprintf(name, 50, "/gnome-mud/Connection%d/Pass=", i);
-			wd->password = gnome_config_private_get_string(name);
-			
-			g_snprintf(name, 50, "/gnome-mud/Connection%d/Profile=", i);
-			wd->profile = gnome_config_get_string(name);
-
-			Profiles = g_list_append(Profiles, (gpointer) wd);
-
-			i++;
-		}
-	}	
-}
-
-void profiledata_save(gchar *profilename, GtkCList *clist, gchar *partname)
-{
-	gchar name[50], tmp[50];
-	gint  clist_count = get_size(clist), i;
-
-	gchar const *vector_name[clist_count];
-	gchar const *vector_value[clist_count];
-
-	gchar *n, *v;
+		WIZARD_DATA2 *wd = g_malloc0(sizeof(WIZARD_DATA2));
+		gchar buf[1024];
 	
-	g_snprintf(name, 50, "/gnome-mud/Profile: %s", profilename);
-
-	for (i = 0; i < clist_count; i++)
-	{
-		gtk_clist_get_text(clist, i, 0, &n);
-		gtk_clist_get_text(clist, i, 1, &v);
-
-		vector_name[i] = n;
-		vector_value[i] = v;
+		LOAD("name", wd->name);
+		LOAD("hostname", wd->hostname);
+		LOAD("port", wd->port);
+		LOAD("playername", wd->playername);
+		LOAD("password", wd->password);
+		LOAD("profile", wd->profile);
+		
+		Profiles = g_list_append(Profiles, (gpointer) wd);
 	}
 
-	g_snprintf(tmp, 50, "%s/%s", name, partname);
-	gnome_config_set_vector(tmp, i, vector_name);
-
-	g_snprintf(tmp, 50, "%s/%sValue", name, partname);
-	gnome_config_set_vector(tmp, i, vector_value);
-}
-
-void profiledata_savekeys(gchar *profilename, KEYBIND_DATA *kd)
-{
-	KEYBIND_DATA *scroll;
-
-	/*
-	 * Ygly hardcoded value
-	 */
-	gchar const *vector_name[500];
-	gchar const *vector_value[500];
-	gchar *buf;
-	gchar tmp[60], name[60];
-	int   i;
-
-	g_snprintf(name, 50, "/gnome-mud/Profile: %s", profilename);
-
-	for (i = 0, scroll = kd; scroll != NULL; i++, scroll = scroll->next)
-	{
-		buf = (char *) g_malloc0(sizeof(char *) * 30);
-		if ((scroll->state)&4)	strcat(buf, "Control+");
-		if ((scroll->state)&8)	strcat(buf, "Alt+");
-		strcat(buf, gdk_keyval_name(scroll->keyv));
-
-		vector_name[i]  = buf;
-		vector_value[i] = scroll->data;
-
-	}
-
-	g_snprintf(tmp, 50, "%s/Keybinds", name);
-	gnome_config_set_vector(tmp, i, vector_name);
-
-	g_snprintf(tmp, 50, "%s/KeybindValues", name);
-	gnome_config_set_vector(tmp, i, vector_value);
+#undef LOAD
 }
 
 PROFILE_DATA *profiledata_find(gchar *profile)
@@ -298,35 +229,48 @@ PROFILE_DATA *profiledata_find(gchar *profile)
 	return NULL;
 }
 
-static void profilelist_new_profile(gchar *string, gpointer data)
+static void profilelist_new_profile(const gchar *string, gpointer data)
 {
-	PROFILE_DATA *pd;
-	gchar *text[1];
 	GList *t;
+	PROFILE_DATA *pd;
+	gchar *pdir, *key, *text[2];
 
 	if (string == NULL)
 	{
 		return;
 	}
 	
-	for (t = g_list_first(ProfilesList); t != NULL; t = t->next)
+	for (t = g_list_first(ProfilesData); t != NULL; t = g_list_next(t))
 	{
-		if (!g_strcasecmp(string, (gchar *) t->data))
+		if (!g_strcasecmp(string, ((PROFILE_DATA *) t->data)->name))
 		{
 			return;
 		}
 	}
-	
-	text[0] = string;
+
+	pdir = gconf_concat_dir_and_key("/apps/gnome-mud/profiles", string);
+
+#define SET_LIST(name) \
+	key = gconf_concat_dir_and_key(pdir, name); \
+	gconf_client_set_list(gconf_client, key, GCONF_VALUE_STRING, NULL, NULL); \
+	g_free(key)
+
+	SET_LIST("aliases");
+	SET_LIST("variables");
+	SET_LIST("triggers");
+	SET_LIST("keybinds");
+
+	text[0] = g_strdup(string);
 	gtk_clist_append(GTK_CLIST(data), text);
-	ProfilesList = g_list_append(ProfilesList, g_strdup(string));
 
 	pd = g_malloc0(sizeof(PROFILE_DATA));
-	pd->name 		= g_strdup(string);
-	pd->alias		= (GtkCList *) gtk_clist_new(2);
-	pd->variables	= (GtkCList *) gtk_clist_new(2);
-	pd->triggers	= (GtkCList *) gtk_clist_new(2);
+	pd->name = g_strdup(string);
 	ProfilesData = g_list_append(ProfilesData, (gpointer) pd);
+
+	profile_gconf_sync_list();
+
+	g_free(text[0]);
+	g_free(pdir);
 }
 
 static void profilelist_new_cb(GtkWidget *widget, gpointer data)
@@ -379,13 +323,19 @@ static void profilelist_delete_cb(GtkWidget *widget, gpointer data)
 		{
 			GList *remove;
 
-			remove = g_list_find_custom(ProfilesList, name, (GCompareFunc) g_list_compare_strings);
-			if (remove != NULL)
+			for (remove = g_list_first(ProfilesData); remove != NULL; remove = g_list_next(remove))
 			{
-				ProfilesList = g_list_remove_link(ProfilesList, remove);
-				g_list_free_1(remove);
+				if (!g_strncasecmp(name, ((PROFILE_DATA *) remove->data)->name, strlen(name)))
+				{
+					ProfilesData = g_list_remove_link(ProfilesData, remove);
+					
+					g_free(((PROFILE_DATA *) remove->data)->name);
+					g_free(remove->data);
+					g_list_free_1(remove);
+					break;
+				}
 			}
-			
+
 			gtk_clist_remove(GTK_CLIST(data), selected);
 		}
 		else
@@ -393,6 +343,8 @@ static void profilelist_delete_cb(GtkWidget *widget, gpointer data)
 			g_message("this profile is in use");
 		}
 	}
+
+	profile_gconf_sync_list();
 }
 
 static void profilelist_alias_cb(GtkWidget *widget, gpointer data)
@@ -504,11 +456,11 @@ static void profilelist_select_row_cb(GtkCList *clist, gint row, gint column, Gd
 	}
 }
 
-static void profilelist_clist_fill(gchar *profile_name, GtkCList *clist)
+static void profilelist_clist_fill(PROFILE_DATA *pd, GtkCList *clist)
 {
 	gchar *text[1];
 	
-	text[0] = profile_name;
+	text[0] = pd->name;
 	gtk_clist_append(clist, text);
 }
 
@@ -532,8 +484,7 @@ static void profilelist_dialog (GtkWidget *label)
 
 	if (dialog != NULL)
 	{
-		gdk_window_raise(dialog->window);
-		gdk_window_show(dialog->window);
+		gtk_window_present (GTK_WINDOW (dialog));
 		return;
 	}
 
@@ -564,7 +515,7 @@ static void profilelist_dialog (GtkWidget *label)
 	gtk_clist_set_column_width (GTK_CLIST (clist), 0, 80);
 	gtk_clist_column_titles_hide (GTK_CLIST (clist));
 	gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_ETCHED_IN);
-	g_list_foreach(ProfilesList, (GFunc) profilelist_clist_fill, (gpointer) clist);
+	g_list_foreach(ProfilesData, (GFunc) profilelist_clist_fill, (gpointer) clist);
 
 	dialog_action_area = GNOME_DIALOG (dialog)->action_area;
 	gtk_object_set_data (GTK_OBJECT (dialog), "dialog_action_area", dialog_action_area);
@@ -591,16 +542,16 @@ static void profilelist_dialog (GtkWidget *label)
 	gtk_signal_connect(GTK_OBJECT(clist), "unselect-row", GTK_SIGNAL_FUNC(profilelist_unselect_row_cb), GINT_TO_POINTER(1));
 
 	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked", GTK_SIGNAL_FUNC(profilelist_button_ok_cb), (gpointer) label);
-	gtk_signal_connect_object(GTK_OBJECT(button_ok), "clicked", gtk_widget_destroy, (gpointer) dialog);
-	gtk_signal_connect_object(GTK_OBJECT(button_cancel), "clicked", gtk_widget_destroy, (gpointer) dialog);
+	gtk_signal_connect_object(GTK_OBJECT(button_ok), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog);
+	gtk_signal_connect_object(GTK_OBJECT(button_cancel), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog);
 
-	gtk_signal_connect(GTK_OBJECT(label->parent->parent), "destroy", profilelist_cleanup, (gpointer) dialog);
+	gtk_signal_connect(GTK_OBJECT(label->parent->parent), "destroy", GTK_SIGNAL_FUNC(profilelist_cleanup), (gpointer) dialog);
 	gtk_signal_connect(GTK_OBJECT(dialog), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
 	
 	gtk_widget_show(dialog);
 }
 
-WIZARD_DATA2 *connections_find(gchar *name, gchar *character)
+WIZARD_DATA2 *connections_find(const gchar *name, const gchar *character)
 {
 	GList *t;
 
@@ -647,8 +598,8 @@ static void connections_button_info_apply_cb(GtkButton *button, GtkCList *clist)
 {
 	gchar *text[2];
 	gchar *label;
-	gchar *name	 = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(button), "entry_info_mud_title")));
-	gchar *charc = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(button), "entry_info_char_character")));
+	const gchar *name  = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(button), "entry_info_mud_title")));
+	const gchar *charc = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(button), "entry_info_char_character")));
 	WIZARD_DATA2 *wd = connections_find(name, charc);
 
 	gboolean new_connection = TRUE;
@@ -690,12 +641,18 @@ static void connections_button_info_apply_cb(GtkButton *button, GtkCList *clist)
 
 	if (new_connection)
 	{
-		text[0] = name;
-		text[1] = charc;
+		text[0] = g_strdup(name);
+		text[1] = g_strdup(charc);
 		gtk_clist_select_row(clist, gtk_clist_append(clist, text), 0);
 
 		Profiles = g_list_append(Profiles, (gpointer) wd);
+
+		g_free(text[0]);
+		g_free(text[1]);
 	}
+
+	profile_gconf_sync_wizard(wd);
+	profile_gconf_sync_connection_list();
 }
 
 static void connections_fields_clean(GtkWidget *widget)
@@ -742,7 +699,7 @@ static void connections_button_connect_cb(GtkButton *button, gpointer data)
 			{
 				connection_send(cd, wd->playername);
 				connection_send(cd, "\n");
-				connection_send(cd, wd->password);
+				connection_send_secret(cd, wd->password);
 				connection_send(cd, "\n");
 			}
 
@@ -764,6 +721,7 @@ static void connections_delete_menu_cb(GtkButton *button, gpointer data)
 		Profiles = g_list_remove(Profiles, (gpointer) wd);
 
 		gtk_clist_remove(GTK_CLIST(main_clist), connection_selected);
+		profile_gconf_sync_connection_list();
 	}
 }
 
@@ -805,7 +763,7 @@ static void connections_unselect_row_cb(GtkCList *clist, gint row, gint column,
 	connection_selected = -1;
 }
 
-static void connections_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
+static gboolean connections_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	gint  row, col;
 	gchar *row_name = NULL;
@@ -832,7 +790,7 @@ static void connections_button_press_cb(GtkWidget *widget, GdkEventButton *event
 					{
 						connection_send(cd, wd->playername);
 						connection_send(cd, "\n");
-						connection_send(cd, wd->password);
+						connection_send_secret(cd, wd->password);
 						connection_send(cd, "\n");
 					}
 
@@ -845,6 +803,8 @@ static void connections_button_press_cb(GtkWidget *widget, GdkEventButton *event
 		default:
 			break;
 	}
+
+	return FALSE;
 }
 
 void window_profiles(void)
@@ -890,8 +850,7 @@ void window_profiles(void)
 
 	if (dialog != NULL)
 	{
-		gdk_window_raise(dialog->window);
-		gdk_window_show(dialog->window);
+		gtk_window_present (GTK_WINDOW (dialog));
 		return;
 	}
  
@@ -1100,22 +1059,19 @@ void window_profiles(void)
 	gtk_container_set_border_width (GTK_CONTAINER (table_buttons), 5);
 	gtk_table_set_row_spacings (GTK_TABLE (table_buttons), 2);
 	gtk_table_set_col_spacings (GTK_TABLE (table_buttons), 3);
-
-	button_info_apply = gnome_stock_button (GNOME_STOCK_BUTTON_APPLY);
-	gtk_widget_ref (button_info_apply);
-	gtk_object_set_data_full (GTK_OBJECT (dialog), "button_info_apply", button_info_apply, (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (button_info_apply);
-	gtk_table_attach (GTK_TABLE (table_buttons), button_info_apply, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 5, 0);
-	gtk_signal_connect(GTK_OBJECT(button_info_apply), "clicked", GTK_SIGNAL_FUNC(connections_button_info_apply_cb), (gpointer) main_clist);
 	
-	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_mud_title", entry_info_mud_title);
-	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_mud_host",  entry_info_mud_host);
-	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_mud_port",  entry_info_mud_port);
-	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_char_character", entry_info_char_character);
-	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_char_password", entry_info_char_password);
-	gtk_object_set_data(GTK_OBJECT(button_info_apply), "label_info_prof", label_info_prof);
+	button_info_fetch = gtk_button_new_with_label(_("Fetch from mudlist"));
+	gtk_widget_show(button_info_fetch);
+	gtk_table_attach(GTK_TABLE(table_buttons), button_info_fetch, 0, 1, 0, 1, GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 0);
+	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_mud_title", entry_info_mud_title);
+	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_mud_host",  entry_info_mud_host);
+	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_mud_port",  entry_info_mud_port);
+	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_char_character", entry_info_char_character);
+	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_char_password", entry_info_char_password);
+	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "label_info_prof",      label_info_prof);
+	gtk_signal_connect(GTK_OBJECT(button_info_fetch), "clicked", GTK_SIGNAL_FUNC(connections_button_info_fetch_cb), NULL);
 
-	button_info_cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
+	button_info_cancel = gtk_button_new_from_stock("gtk-cancel");
 	gtk_widget_ref (button_info_cancel);
 	gtk_object_set_data_full (GTK_OBJECT (dialog), "button_info_cancel", button_info_cancel, (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (button_info_cancel);
@@ -1129,16 +1085,19 @@ void window_profiles(void)
 	gtk_object_set_data(GTK_OBJECT(button_info_cancel), "entry_info_char_password", entry_info_char_password);
 	gtk_object_set_data(GTK_OBJECT(button_info_cancel), "label_info_prof", label_info_prof);
 
-	button_info_fetch = gtk_button_new_with_label(_("Fetch from mudlist"));
-	gtk_widget_show(button_info_fetch);
-	gtk_table_attach(GTK_TABLE(table_buttons), button_info_fetch, 2, 3, 0, 1, GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 0);
-	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_mud_title", entry_info_mud_title);
-	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_mud_host",  entry_info_mud_host);
-	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_mud_port",  entry_info_mud_port);
-	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_char_character", entry_info_char_character);
-	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "entry_info_char_password", entry_info_char_password);
-	gtk_object_set_data(GTK_OBJECT(button_info_fetch), "label_info_prof",      label_info_prof);
-	gtk_signal_connect(GTK_OBJECT(button_info_fetch), "clicked", GTK_SIGNAL_FUNC(connections_button_info_fetch_cb), NULL);
+	button_info_apply = gtk_button_new_from_stock ("gtk-apply");
+	gtk_widget_ref (button_info_apply);
+	gtk_object_set_data_full (GTK_OBJECT (dialog), "button_info_apply", button_info_apply, (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (button_info_apply);
+	gtk_table_attach (GTK_TABLE (table_buttons), button_info_apply, 2, 3, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 5, 0);
+	gtk_signal_connect(GTK_OBJECT(button_info_apply), "clicked", GTK_SIGNAL_FUNC(connections_button_info_apply_cb), (gpointer) main_clist);
+	
+	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_mud_title", entry_info_mud_title);
+	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_mud_host",  entry_info_mud_host);
+	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_mud_port",  entry_info_mud_port);
+	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_char_character", entry_info_char_character);
+	gtk_object_set_data(GTK_OBJECT(button_info_apply), "entry_info_char_password", entry_info_char_password);
+	gtk_object_set_data(GTK_OBJECT(button_info_apply), "label_info_prof", label_info_prof);
 
 	dialog_action_area = GNOME_DIALOG (dialog)->action_area;
 	gtk_object_set_data (GTK_OBJECT (dialog), "dialog_action_area", dialog_action_area);
@@ -1146,14 +1105,6 @@ void window_profiles(void)
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_END);
 	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area), 8);
  
-	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog), _("Connect"), GNOME_STOCK_PIXMAP_JUMP_TO);
-	button_connect = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog)->buttons)->data);
-	gtk_widget_ref (button_connect);
-	gtk_object_set_data_full (GTK_OBJECT (dialog), "button_connect", button_connect, (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (button_connect);
-	GTK_WIDGET_SET_FLAGS (button_connect, GTK_CAN_DEFAULT);
-	gtk_signal_connect(GTK_OBJECT(button_connect), "clicked", GTK_SIGNAL_FUNC(connections_button_connect_cb), (gpointer) dialog);
-
 	gnome_dialog_append_button (GNOME_DIALOG (dialog), GNOME_STOCK_BUTTON_CLOSE);
 	button_cancel = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog)->buttons)->data);
 	gtk_widget_ref (button_cancel);
@@ -1161,7 +1112,15 @@ void window_profiles(void)
 	gtk_widget_show (button_cancel);
 	GTK_WIDGET_SET_FLAGS (button_cancel, GTK_CAN_DEFAULT);
   
-	gtk_signal_connect_object(GTK_OBJECT(button_cancel), "clicked", gtk_widget_destroy, (gpointer) dialog);
+  	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog), _("Connect"), GNOME_STOCK_PIXMAP_JUMP_TO);
+	button_connect = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog)->buttons)->data);
+	gtk_widget_ref (button_connect);
+	gtk_object_set_data_full (GTK_OBJECT (dialog), "button_connect", button_connect, (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (button_connect);
+	GTK_WIDGET_SET_FLAGS (button_connect, GTK_CAN_DEFAULT);
+	gtk_signal_connect(GTK_OBJECT(button_connect), "clicked", GTK_SIGNAL_FUNC(connections_button_connect_cb), (gpointer) dialog);
+
+	gtk_signal_connect_object(GTK_OBJECT(button_cancel), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog);
 	gtk_signal_connect(GTK_OBJECT(dialog), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
 
 	gtk_object_set_data(GTK_OBJECT(main_clist), "entry_info_mud_title", entry_info_mud_title);
@@ -1178,7 +1137,7 @@ void window_profiles(void)
 
 void window_profile_edit(void)
 {
-	static GtkWidget *profile_window;
+	static GtkWidget *profile_window = NULL;
 	GtkWidget *vbox;
 	GtkWidget *toolbar;
 	GtkWidget *tmp_toolbar_icon;
@@ -1196,8 +1155,7 @@ void window_profile_edit(void)
 
 	if (profile_window != NULL)
 	{
-		gdk_window_raise(profile_window->window);
-		gdk_window_show(profile_window->window);
+		gtk_window_present (GTK_WINDOW (profile_window));
 		return;
 	}
 	
@@ -1208,25 +1166,23 @@ void window_profile_edit(void)
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(profile_window), vbox);
 
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);
+	toolbar = gtk_toolbar_new ();
 	gtk_widget_show(toolbar);
 	gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);
 	
 	/*
 	 * Button new
 	 */
-	tmp_toolbar_icon = gnome_stock_pixmap_widget(profile_window, GNOME_STOCK_PIXMAP_NEW);
-	button_new = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL, _("New"), NULL, NULL,
-										    tmp_toolbar_icon, NULL, NULL);
+	tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-new", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	button_new = gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("New"),
+			_("Create a new profile"), NULL, tmp_toolbar_icon, NULL,NULL);
 	gtk_widget_show(button_new);
 
 	/*
 	 * Button delete
 	 */
-	tmp_toolbar_icon = gnome_stock_pixmap_widget(profile_window, GNOME_STOCK_PIXMAP_TRASH);
-	button_delete = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Delete"), NULL, NULL,
-											   tmp_toolbar_icon, NULL, NULL);
+	tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-new", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	button_delete = gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("Delete"), _("Delete a profile"), NULL, tmp_toolbar_icon, NULL, NULL);
 	gtk_widget_show(button_delete);
 
 	/*
@@ -1242,25 +1198,22 @@ void window_profile_edit(void)
 	/*
 	 * Button alias
 	 */
-	tmp_toolbar_icon = gnome_stock_pixmap_widget(profile_window, GNOME_STOCK_PIXMAP_PREFERENCES);
-	button_alias = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Aliases"), NULL, NULL,
-											  tmp_toolbar_icon, NULL, NULL);
+	tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-preferences", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	button_alias = gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("Aliases"), _("Set aliases"), NULL, tmp_toolbar_icon, NULL, NULL);
 	gtk_widget_show(button_alias);
 
 	/*
 	 * Button variables
 	 */
-	tmp_toolbar_icon = gnome_stock_pixmap_widget(profile_window, GNOME_STOCK_PIXMAP_SPELLCHECK);
-	button_variables = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Variables"), NULL, NULL,
-												  tmp_toolbar_icon, NULL, NULL);
+	tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-spell-check", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	button_variables = gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("Variables"), _("Set variables"), NULL, tmp_toolbar_icon, NULL, NULL);
 	gtk_widget_show(button_variables);
 
 	/*
 	 * Button triggers
 	 */
-	tmp_toolbar_icon = gnome_stock_pixmap_widget(profile_window, GNOME_STOCK_PIXMAP_INDEX);
-	button_triggers = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Triggers"), NULL, NULL,
-												 tmp_toolbar_icon, NULL, NULL);
+	tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-index", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	button_triggers = gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("Triggers"), _("Set triggers"), NULL, tmp_toolbar_icon, NULL, NULL);
 	gtk_widget_show(button_triggers);
 
 	/*
@@ -1276,9 +1229,8 @@ void window_profile_edit(void)
 	/*
 	 * Button keybinds
 	 */
-	tmp_toolbar_icon = gnome_stock_pixmap_widget(profile_window, GNOME_STOCK_PIXMAP_MIDI);
-	button_keybinds = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Keybinds"), NULL, NULL,
-												 tmp_toolbar_icon, NULL, NULL);
+	tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-select-font", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	button_keybinds = gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), _("Keybinds"), _("Set keybinds"), NULL, tmp_toolbar_icon, NULL, NULL);
 	gtk_widget_show(button_keybinds);
 
 	/*
@@ -1294,10 +1246,10 @@ void window_profile_edit(void)
 	/*
 	 *
 	 */
-	tmp_toolbar_icon = gnome_stock_pixmap_widget(profile_window, GNOME_STOCK_PIXMAP_CLOSE);
-	button_close = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Close"), NULL, NULL,
-											  tmp_toolbar_icon, NULL, NULL);
+	tmp_toolbar_icon = gtk_image_new_from_stock("gtk-close", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	button_close = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), _("Close"), _("Close the window"), NULL, tmp_toolbar_icon, NULL, NULL);
 	gtk_widget_show(button_close);
+	
 	/*
 	 * CList
 	 */
@@ -1314,13 +1266,13 @@ void window_profile_edit(void)
 	gtk_clist_column_title_passive(GTK_CLIST(profile_list), 0);
 	gtk_object_set_data(GTK_OBJECT(profile_list), "main_window", profile_window);
 
-	g_list_foreach(ProfilesList, (GFunc) profilelist_clist_fill, (gpointer) profile_list);
+	g_list_foreach(ProfilesData, (GFunc) profilelist_clist_fill, (gpointer) profile_list);
 
 	/*
 	 * Signals
 	 */
-	gtk_signal_connect(GTK_OBJECT(profile_list), "select-row",   		GTK_SIGNAL_FUNC(profilelist_select_row_cb), GINT_TO_POINTER(0));
-	gtk_signal_connect(GTK_OBJECT(profile_list), "unselect-row", 		GTK_SIGNAL_FUNC(profilelist_unselect_row_cb), GINT_TO_POINTER(0));
+	gtk_signal_connect(GTK_OBJECT(profile_list), "select-row", GTK_SIGNAL_FUNC(profilelist_select_row_cb), GINT_TO_POINTER(0));
+	gtk_signal_connect(GTK_OBJECT(profile_list), "unselect-row", GTK_SIGNAL_FUNC(profilelist_unselect_row_cb), GINT_TO_POINTER(0));
 
 	gtk_signal_connect(GTK_OBJECT(button_new),    "clicked", GTK_SIGNAL_FUNC(profilelist_new_cb), profile_list);
 	gtk_signal_connect(GTK_OBJECT(button_delete), "clicked", GTK_SIGNAL_FUNC(profilelist_delete_cb), profile_list);
@@ -1328,10 +1280,9 @@ void window_profile_edit(void)
 	gtk_signal_connect(GTK_OBJECT(button_alias),     "clicked", GTK_SIGNAL_FUNC(profilelist_alias_cb), profile_list);
 	gtk_signal_connect(GTK_OBJECT(button_triggers),  "clicked", GTK_SIGNAL_FUNC(profilelist_triggers_cb), profile_list);
 	gtk_signal_connect(GTK_OBJECT(button_variables), "clicked", GTK_SIGNAL_FUNC(profilelist_variables_cb), profile_list);
-
 	gtk_signal_connect(GTK_OBJECT(button_keybinds),  "clicked", GTK_SIGNAL_FUNC(profilelist_keybinds_cb), profile_list);
 
-	gtk_signal_connect_object(GTK_OBJECT(button_close), "clicked", gtk_widget_destroy, (gpointer) profile_window);
+	gtk_signal_connect_object(GTK_OBJECT(button_close), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) profile_window);
 	gtk_signal_connect(GTK_OBJECT(profile_window), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &profile_window);
 	
 	gtk_widget_show(profile_window);
